/*
 * Copyright (C) 2014-2016 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <errno.h>
#include <libmockturtle.h>


/**
 * It prints the help message
 * @param[in] name the program name
 */
static void help(char *name)
{
	fprintf(stderr, "\n");
	fprintf(stderr, "%s [options]\n\n", name);
	fprintf(stderr, "It shows the Mock Turtle devices available on the system.\n\n");
	fprintf(stderr, "-v   show more information\n");
	fprintf(stderr, "-h   show this help\n");
	fprintf(stderr, "\n");
	exit(1);
}


/**
 * It shows information about a given Mock Turtle device
 * @param[in] trtl Mock Turtle device instance
 */
static void trtl_list_device_info(struct trtl_dev *trtl)
{
	unsigned int appid = 0, cpucount = 0;

	trtl_app_id_get(trtl, &appid);
	trtl_cpu_count(trtl, &cpucount);
	fprintf(stdout, "    Application ID: 0x%08x\n", appid);
	fprintf(stdout, "    Number of CPU: %d\n", cpucount);
}


int main(int argc, char *argv[])
{
	char **list;
	struct trtl_dev *trtl;
	char c;
	int i, verbose = 0, err;

	atexit(trtl_exit);

	while ((c = getopt (argc, argv, "h:v")) != -1) {
		switch (c) {
		default:
			help(argv[0]);
			break;
		case 'v':
			verbose++;
			break;
		}
	}

	err = trtl_init();
	if (err) {
		fprintf(stderr, "Cannot init White Rabbit Node Core lib: %s\n",
			trtl_strerror(errno));
		exit(1);
	}

	list = trtl_list();
	if (!list)
		goto out;
	for (i = 0; list[i]; ++i) {
		fprintf(stdout, "%s\n" , list[i]);
		trtl = trtl_open(list[i]);
		if (!trtl) {
			fprintf(stderr, "Cannot open device: %s\n",
				trtl_strerror(errno));
			continue;
		}

		switch (verbose) {
		case 1:
			trtl_list_device_info(trtl);
			break;
		}
		trtl_close(trtl);
	}
	trtl_list_free(list);
out:
	exit(0);
}
